//GSP480 - Monte Carlo Lab
//Tylor Emmett
//4/22/15

#include "Dealer.h"
#include <iostream>

Dealer::Dealer()
{
	isStanding = false;
}

Dealer* Dealer::Instance()
{
	static Dealer instance;
	return &instance;
}

void Dealer::PrintHand(bool hideSecondCard)
{
	for (unsigned i = 0; i < hand.size(); i++)
	{
		if ((hideSecondCard) && (i = 1))
			std::cout << "X ";
		Cards::PrintCard(hand[i]);
	}
	std::cout << std::endl;
}

void Dealer::TakeTurn()
{
	//Skip remainder if standing
	if (isStanding)
		return;

	//Hit if under 17
	if (HandValue() < 17)
	{
		hand.emplace_back(Deck::Instance()->DrawCard());
		std::cout << "Dealer Hits" << std::endl;
	}
	//Other wise stand
	else
	{
		std::cout << "Dealer Stands" << std::endl;
		isStanding = true;
	}

}

void Dealer::GiveCard(Cards::Card cardIn)
{
	hand.emplace_back(cardIn);
}

bool Dealer::IsStanding()
{
	return isStanding;
}

void Dealer::Reset()
{
	isStanding = false;
	hand.clear();
}

int Dealer::HandValue()
{
	//Find hand value
	int numAces = 0;
	int handValue = 0;
	auto itr = hand.begin();
	for (; itr != hand.end(); itr++)
	{
		if (*itr == Cards::Card::A)
		{
			numAces++;
		}

		//Add enum value + 1 for cards A through 10 (which are 0 through 9)
		if (*itr < 10)
		{
			handValue += *itr + 1;
		}
		//Add 10 for the face cards
		else
		{
			handValue += 10;
		}
	}

	//If value is below 11, add 10 per ace until the total breaks 11 (which would cause a bust if using an ace as 11)
	while ((handValue <= 11) && (numAces > 0))
	{
		handValue += 10;
		numAces--;
	}

	return handValue;
}
