//GSP480 - Monte Carlo Lab
//Tylor Emmett
//4/22/15

#include "Dealer.h"
#include "Player.h"
#include "MonteCarlo.h"
#include <iostream>
#include <string>

void GameStart();

void GameRun();

void GameOver();

int main()
{
	while (true)
	{
		GameStart();
		GameRun();
		GameOver();

		//Note: added the option to play again to simplify the testing process
		std::string buffer;
		std::cout << "Play Again? (Y)es/(N)o" << std::endl;
		//Check first letter of input for an n to quit
		getline(std::cin, buffer);
		tolower(buffer[0]);
		if (buffer.substr(0, 1) == "n")
			break;
	}

}

void GameStart()
{
	//Reset both players and deal them each two cards in sequence (player, dealer, player, dealer)
	Dealer::Instance()->Reset();
	Player::Instance()->Reset();
	Player::Instance()->GiveCard(Deck::Instance()->DrawCard());
	Dealer::Instance()->GiveCard(Deck::Instance()->DrawCard());
	Player::Instance()->GiveCard(Deck::Instance()->DrawCard());
	Dealer::Instance()->GiveCard(Deck::Instance()->DrawCard());
	
}


void GameRun()
{
	while (true)
	{
		//Break loop in case of blackjack
		if ((Player::Instance()->HandValue() == 21) || (Dealer::Instance()->HandValue() == 21))
			break;

		//Clear output window
		system("cls");

		//Print Current Hands
		if (Player::Instance()->IsStanding())
			std::cout << "Dealer Hand: " << Dealer::Instance()->HandValue() << std::endl;
		else
			std::cout << "Dealer Hand: " << "?" << std::endl;
		Dealer::Instance()->PrintHand(!Player::Instance()->IsStanding()); //If player hasn't finsihed turn (is not standing), one card will be an hidden.
		std::cout << std::endl;
		std::cout << "Player Hand: " << Player::Instance()->HandValue() << std::endl;
		Player::Instance()->PrintHand();

		//Player plays until standing
		if (!Player::Instance()->IsStanding())
		{
			Player::Instance()->TakeTurn();
		}

		//Dealer plays until standing
		else if (!Dealer::Instance()->IsStanding())
		{
			Dealer::Instance()->TakeTurn();
		}
		//Break the loop if both stand.
		else
		{
			break;
		}
		
	}
}

void GameOver()
{
	//Print final hands and announce a winner.
	system("cls");
	int dealerHand = Dealer::Instance()->HandValue();
	int playerHand = Player::Instance()->HandValue();
	std::cout << "Dealer Hand: " << dealerHand << std::endl;
	Dealer::Instance()->PrintHand(false);
	std::cout << std::endl;
	std::cout << "Player Hand: " << playerHand << std::endl;
	Player::Instance()->PrintHand();
	std::cout << std::endl;

	if (((dealerHand > playerHand) || (playerHand > 21)) && (dealerHand < 22))
	{
		std::cout << "Dealer Wins!" << std::endl;
	}
	else if (((playerHand > dealerHand) || (dealerHand > 21)) && (playerHand < 22))
	{
		std::cout << "Player Wins!" << std::endl;
	}
	else
	{
		std::cout << "Game is Tied!" << std::endl;
	}
}
