//GSP480 - Monte Carlo Lab
//Tylor Emmett
//4/22/15

#include "MonteCarlo.h"
#include "Dealer.h"
#include "Player.h"

MonteCarlo::MonteCarlo()
{
	//Get the player and dealer hand/stats
	playerStartingHand = Player::Instance()->hand;
	dealerStartingHand = Dealer::Instance()->hand;
	startingPlayerStanding = Player::Instance()->isStanding;
	startingDealerStanding = Dealer::Instance()->isStanding;
	//Start logfile
	MonteCarloLog.open("Log.txt", std::ios::app);
}

MonteCarlo* MonteCarlo::Instance()
{
	static MonteCarlo instance;
	return &instance;
}

void MonteCarlo::Reset()
{
	//Get the dealer and player hand/stats
	playerStartingHand = Player::Instance()->hand;
	dealerStartingHand = Dealer::Instance()->hand;
	startingPlayerStanding = Player::Instance()->isStanding;
	startingDealerStanding = Dealer::Instance()->isStanding;
}

MonteCarloResults MonteCarlo::RunSimulation()
{
	MonteCarloResults output;
	float hitWins = 0.f;
	float hitDraws = 0.f;
	float hitLosses = 0.f;
	float standWins = 0.f;
	float standDraws = 0.f;
	float standLosses = 0.f;

	//Hit first then stand
	for (int i = 0; i < 1000; i++)
	{
		//Log header
		MonteCarloLog << "Player Hits - Iteration: " << i << std::endl;

		//Play through turns
		PlayerPlay(true);
		DealerPlay();

		//Log results
		MonteCarloLog << "Player Hand: ";
		for (unsigned j = 0; j < Player::Instance()->hand.size(); j++)
		{
			MonteCarloLog << " " << Player::Instance()->hand[j];
		}

		MonteCarloLog << std::endl << "Dealer Hand: ";
		for (unsigned j = 0; j < Dealer::Instance()->hand.size(); j++)
		{
			MonteCarloLog << " " << Dealer::Instance()->hand[j];
		}
		MonteCarloLog << std::endl;

		//Check win/loss
		int dealerHand = Dealer::Instance()->HandValue();
		int playerHand = Player::Instance()->HandValue();
		
		if (((dealerHand > playerHand) || (playerHand > 21)) && (dealerHand < 22))
		{
			hitLosses++;
			
		}
		else if (((playerHand > dealerHand) || (dealerHand > 21)) && (playerHand < 22))
		{
			hitWins++;
		}
		else
		{
			hitDraws++;
		}

		//Log win/loss/draw
		MonteCarloLog << "Wins: " << hitWins << std::endl << "Draws: " << hitDraws << std::endl << "Losses: " << hitLosses << std::endl;

		//Restore player/dealer hands/standing
		Player::Instance()->hand = playerStartingHand;
		Player::Instance()->isStanding = startingPlayerStanding;
		Dealer::Instance()->hand = dealerStartingHand;
		Dealer::Instance()->isStanding = startingDealerStanding;
	}

	//Stand first
	for (int i = 0; i < 1000; i++)
	{
		MonteCarloLog << "Player Stands - Iteration: " << i << std::endl;
		PlayerPlay(false);
		DealerPlay();

		//Log results
		MonteCarloLog << "Player Hand: ";
		for (unsigned j = 0; j < Player::Instance()->hand.size(); j++)
		{
			MonteCarloLog << " " << Player::Instance()->hand[j];
		}

		MonteCarloLog << std::endl << "Dealer Hand: ";
		for (unsigned j = 0; j < Dealer::Instance()->hand.size(); j++)
		{
			MonteCarloLog << " " << Dealer::Instance()->hand[j];
		}
		MonteCarloLog << std::endl;

		//Check wins/losses
		int dealerHand = Dealer::Instance()->HandValue();
		int playerHand = Player::Instance()->HandValue();
		if (((dealerHand > playerHand) || (playerHand > 21)) && (dealerHand < 22))
		{
			standLosses++;
		}
		else if (((playerHand > dealerHand) || (dealerHand > 21)) && (playerHand < 22))
		{
			standWins++;
		}
		else
		{
			standDraws++;
		}
		MonteCarloLog << "Wins: " << standWins << std::endl << "Draws: " << standDraws << std::endl << "Losses: " << standLosses << std::endl;

		//Restore player/dealer hands/standing
		Player::Instance()->hand = playerStartingHand;
		Player::Instance()->isStanding = startingPlayerStanding;
		Dealer::Instance()->hand = dealerStartingHand;
		Dealer::Instance()->isStanding = startingDealerStanding;
	}


	

	//Prepare and return output
	output.HitDrawChance = hitDraws / 1000.f * 100.f;
	output.HitWinChance = hitWins / 1000.f * 100.f;
	output.HitLossChance = hitLosses / 1000.f * 100.f;
	output.StandDrawChance = standDraws / 1000.f * 100.f;
	output.StandWinChance = standWins / 1000.f * 100.f;
	output.StandLossChance = standLosses / 1000.f * 100.f;
	return output;
}

void MonteCarlo::PlayerPlay(bool hitFirst)
{
	//Draw a card if hitting first
	if (hitFirst)
	{
		Player::Instance()->GiveCard(Deck::Instance()->DrawCard());
	}
	//Stand
	Player::Instance()->isStanding = true;
}

void MonteCarlo::DealerPlay()
{
	//Draw cards while hand is less than 17
	while (Dealer::Instance()->HandValue() < 17)
	{
		Dealer::Instance()->GiveCard(Deck::Instance()->DrawCard());
	}
	//Stand
	Dealer::Instance()->isStanding = true;
}

