//GSP480 - Monte Carlo Lab
//Tylor Emmett
//4/22/15

#ifndef MONTECARLO_H
#define MONTECARLO_H

#include "Deck.h"
#include <vector>
#include <fstream>

//Container for the results of Monte Carlo sim
struct MonteCarloResults
{
	float HitWinChance;
	float HitLossChance;
	float HitDrawChance;
	float StandWinChance;
	float StandLossChance;
	float StandDrawChance;
};
class MonteCarlo
{
	std::ofstream MonteCarloLog;		//Log output for Monte Carlo sim debugging.
	std::vector<Cards::Card> playerStartingHand;	//Starting player hand
	std::vector<Cards::Card> dealerStartingHand;	//Starting dealer hand
	bool startingPlayerStanding;		//Staring player state
	bool startingDealerStanding;		//Staring dealer state
	MonteCarlo();						//Private C-tor for singleton
	void PlayerPlay(bool hitFirst);		//Processes the Player's round. Parameter determines if the player will hit on the first turn
	void DealerPlay();					//Processes the Dealer's round
public:
	static MonteCarlo* Instance();		//Accessor
	void Reset();						//Gets current player data
	MonteCarloResults RunSimulation();	//Runs 1000 sims for both player hit and player stand

	//Deleted copy and assignment for singleton
	MonteCarlo(MonteCarlo&) = delete;
	void operator=(MonteCarlo&) = delete;
};

#endif