//GSP480 - Monte Carlo Lab
//Tylor Emmett
//4/22/15

#include "Player.h"
#include <iostream>
#include <string>
#include "MonteCarlo.h"

Player::Player()
{
	isStanding = false;
}

Player* Player::Instance()
{
	static Player instance;
	return &instance;
}

void Player::PrintHand()
{
	for (unsigned i = 0; i < hand.size(); i++)
	{
		Cards::PrintCard(hand[i]);
	}
	std::cout << std::endl;
}

void Player::TakeTurn()
{
	//Skip remainder if standing
	if (isStanding)
		return;

	//Bust if over 21
	if (HandValue() > 21)
	{
		std::cout << "Player Busts" << std::endl;
		isStanding = true;
		return;
	}

	//Run Monte Carlo sim
	std::cout << "Monte Carlo Statistics for This Hand:" << std::endl;
	MonteCarloResults results;
	MonteCarlo::Instance()->Reset();
	results = MonteCarlo::Instance()->RunSimulation();
	std::cout << "Stand: Win % " << results.StandWinChance << std::endl;
	std::cout << "Stand: Draw % " << results.StandDrawChance << std::endl;
	std::cout << "Stand: Loss % " << results.StandLossChance << std::endl;
	std::cout << "Hit: Win % " << results.HitWinChance << std::endl;
	std::cout << "Hit: Draw % " << results.HitDrawChance << std::endl;
	std::cout << "Hit: Loss % " << results.HitLossChance << std::endl;

	//Get next move
	std::string buffer;
	std::cout << "Would you like to (H)it, (S)tand?" << std::endl;
	getline(std::cin, buffer);
	
	//Checking just the first letter for h or s (in case Hit or Stand is entered instead).
	tolower(buffer[0]);
	if (buffer.substr(0, 1) == "h")
	{
		hand.emplace_back(Deck::Instance()->DrawCard());
	}
	else if (buffer.substr(0, 1) == "s")
	{
		isStanding = true;
	}
}

void Player::GiveCard(Cards::Card cardIn)
{
	hand.emplace_back(cardIn);
}

bool Player::IsStanding()
{
	return isStanding;
}

void Player::Reset()
{
	isStanding = false;
	hand.clear();
}

int Player::HandValue()
{
	//Find hand value
	int numAces = 0;
	int handValue = 0;
	auto itr = hand.begin();
	for (; itr != hand.end(); itr++)
	{
		if (*itr == Cards::Card::A)
		{
			numAces++;
		}

		//Add enum value + 1 for cards A through 10 (which are 0 through 9)
		if (*itr < 10)
		{
			handValue += *itr + 1;
		}
		//Add 10 for the face cards
		else
		{
			handValue += 10;
		}
	}

	//If the hand total is less than 11, add 10 per ace until the total breaks 11 (which would cause a bust if using an ace as 11)
	while ((handValue <= 11) && (numAces > 0))
	{
		handValue += 10;
		numAces--;
	}

	return handValue;
}
